﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Collections.ObjectModel;

namespace ViewModelValidation
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        MainViewModel vm = new MainViewModel();
        public MainWindow()
        {
            InitializeComponent();
            DataContext = vm;
        }
    }

    public class MainViewModel : ViewModelVase
    {
        private string[] shipMethods = new string[]
        {
            "mail",
            "airplane",
            "ship",
            "bus"
        };

        private ObservableCollection<Data> data;

        public MainViewModel()
        {
            DataSource = new ObservableCollection<Data>();
            Random r =  new Random();
            for (int i = 0; i < 10; i++)
            {
                DataSource.Add(new Data
                {
                    Name = i % 7 == 0 ? "i" + i :"Item " + i,
                    Price = i % 5 ==0 ? -5 : r.Next(20) + r.NextDouble(),
                    ShipMethod = i%4 == 0 ? "some" : shipMethods[r.Next(shipMethods.Length)]                 
                });
            }
        }

        public ObservableCollection<Data> DataSource
        {
            get { return data; }
            set { data = value; }
        }

    }

    public class Data : ViewModelVase
    {
        private string[] shipMethods = new string[]
        {
            "mail",
            "airplane",
            "ship",
            "bus"
        };

        private string name;
        private double price;
        private string method;

        public string Name
        {
            get { return name; }
            set
            {
                IsNameValid(value);
                name = value;
                OnPropertyChagned("Name");
            }
           
        }
        public double Price
        {
            get { return price; }
            set
            {
                IsPriceValid(value);

                price = value;
                OnPropertyChagned("Price");
            }
        }



        public string ShipMethod
        {
            get { return method; }
            set 
            {
                IsShipMethodValide(value);

                method = value;
                OnPropertyChagned("ShipMethod");

            }
        }



        #region Validate
        private bool IsPriceValid(double value)
        {
            if (value < 0)
            {
                AddError("Price", "Size must be non-negative");
                return false;
            }
            else
            {
                RemoveError("Price", "Size must be non-negative");
                return true;
            }
        }
        private bool IsNameValid(string name)
        {
            if (string.IsNullOrEmpty(name) || name.Length < 3)
            {
                AddError("Name", "Name must be 3 chars or more");
                return false;
            }
            else
            {
                RemoveError("Name", "Name must be 3 chars or more");
                return true;
            }
        }
        private bool IsShipMethodValide(string method)
        {
            if (!shipMethods.Contains(method))
            {
                AddError("ShipMethod", "Valid Ship Methods are:\nmail,\nairplane,\nbus,\nship");
                return false;
            }
            else
            {
                RemoveError("ShipMethod", "Valid Ship Methods are:\nmail,\nairplane,\nbus,\nship");
                return true;
            }
        }
        #endregion


    }



    public class ViewModelVase : INotifyPropertyChanged, IDataErrorInfo
    {

        #region INotifyPropertyChanged Members
        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChagned(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }
        #endregion


        #region IDataErrorInfo members
        Dictionary<string, List<string>> errors = new Dictionary<string, List<string>>();

        public string Error
        {
            get { return ""; }
        }

        public string this[string columnName]
        {
            get
            {
                return (!errors.ContainsKey(columnName) ? null :
                      String.Join(Environment.NewLine, errors[columnName]));
            }
        }

        //Add errors
        public void AddError(string prop, string err)
        {
            if (!errors.ContainsKey(prop))
                errors.Add(prop, new List<string>());
            if (!errors[prop].Contains(err))
                errors[prop].Add(err);
        }
        //Remove errors
        public void RemoveError(string prop, string err)
        {
            if (errors.ContainsKey(prop) &&
              errors[prop].Contains(err))
            {
                errors[prop].Remove(err);
                if (errors[prop].Count == 0) errors.Remove(prop);
            }
        }
        #endregion
    }
}
